@@ -2,7 +2,7 @@ import ast, sys, json, os, time, base64
 import requests
 import random, copy
 from collections import OrderedDict
-from hwilib.descriptor import AddChecksum, Descriptor
+from .descriptor import AddChecksum, Descriptor
 from mnemonic import Mnemonic
 from threading import Thread
 from .key import Key
@@ -15,7 +15,6 @@ from flask_login import login_required, login_user, logout_user, current_user
 from flask_login.config import EXEMPT_METHODS
 
 
-from .devices.bitcoin_core import BitcoinCore
 from .helpers import (alias, get_devices_with_keys_by_type, hash_password, 
                       get_loglevel, get_version_info, run_shell, set_loglevel, 
                       verify_password, bcur2base64, get_txid, generate_mnemonic,
@@ -233,7 +232,7 @@ def login():
 
 def redirect_login(request):
     flash('Logged in successfully.',"info")
-    if request.form.get('next') and request.form.get('next').startswith("http"):
+    if request.form.get('next'):
         response = redirect(request.form['next'])
     else:
         response = redirect(url_for('index'))
@@ -299,11 +298,15 @@ def settings():
         return redirect("/settings/general")
 
 
-@app.route('/settings/hwi', methods=['GET'])
+@app.route('/settings/hwi', methods=['GET', 'POST'])
 @login_required
 def hwi_settings():
     current_version = notify_upgrade()
     app.specter.check()
+    if request.method == 'POST':
+        hwi_bridge_url = request.form['hwi_bridge_url']
+        app.specter.update_hwi_bridge_url(hwi_bridge_url, current_user)
+        flash("HWIBridge URL is updated! Don't forget to whitelist Specter!")
     return render_template(
         "settings/hwi_settings.jinja",
         specter=app.specter,
@@ -318,12 +321,12 @@ def general_settings():
     current_version = notify_upgrade()
     app.specter.check()
     explorer = app.specter.explorer
-    hwi_bridge_url = app.specter.hwi_bridge_url
     loglevel = get_loglevel(app)
+    unit = app.specter.unit
     if request.method == 'POST':
         action = request.form['action']
         explorer = request.form['explorer']
-        hwi_bridge_url = request.form['hwi_bridge_url']
+        unit = request.form['unit']
         if current_user.is_admin:
             loglevel = request.form['loglevel']
 
@@ -332,7 +335,7 @@ def general_settings():
                 set_loglevel(app, loglevel)
 
             app.specter.update_explorer(explorer, current_user)
-            app.specter.update_hwi_bridge_url(hwi_bridge_url, current_user)
+            app.specter.update_unit(unit, current_user)
             app.specter.check()
         elif action == "backup":
             return send_file(
@@ -424,8 +427,8 @@ This may take a few hours to complete.', 'info')
     return render_template(
         "settings/general_settings.jinja",
         explorer=explorer,
-        hwi_bridge_url=hwi_bridge_url,
         loglevel=loglevel,
+        unit=unit,
         specter=app.specter,
         current_version=current_version,
         rand=rand
@@ -654,7 +657,10 @@ def new_wallet(wallet_type):
             wallet_name = wallet_data['label'] if 'label' in wallet_data else 'Imported Wallet'
             startblock = wallet_data['blockheight'] if 'blockheight' in wallet_data else app.specter.wallet_manager.cli.getblockcount()
             try:
-                descriptor = Descriptor.parse(AddChecksum(wallet_data['descriptor'].split('#')[0]), testnet=app.specter.chain != 'main')
+                descriptor = Descriptor.parse(
+                    AddChecksum(wallet_data['descriptor'].split('#')[0]),
+                    testnet=app.specter.chain != 'main'
+                )
                 if descriptor is None:
                     err = "Invalid wallet descriptor."
             except:
@@ -691,9 +697,18 @@ def new_wallet(wallet_type):
                         cosigner_found = False
                         for device in app.specter.device_manager.devices:
                             cosigner = app.specter.device_manager.devices[device]
+                            if descriptor.origin_fingerprint[i] is None:
+                                descriptor.origin_fingerprint[i] = ''
+                            if descriptor.origin_path[i] is None:
+                                descriptor.origin_path[i] = \
+                                    descriptor.origin_fingerprint[i]
                             for key in cosigner.keys:
-                                if key.fingerprint + key.derivation.replace('m', '') == \
-                                    descriptor.origin_fingerprint[i] + descriptor.origin_path[i].replace("'", 'h'):
+                                if key.fingerprint + \
+                                    key.derivation.replace('m', '') == \
+                                    descriptor.origin_fingerprint[i] + \
+                                    descriptor.origin_path[i].replace(
+                                        "'", 'h'
+                                        ):
                                     keys.append(key)
                                     cosigners.append(cosigner)
                                     cosigner_found = True
@@ -955,6 +970,14 @@ def fees(blocks):
     res = app.specter.estimatesmartfee(int(blocks))
     return res
 
+
+@app.route('/get_txout_set_info')
+@login_required
+def txout_set_info():
+    res = app.specter.cli.gettxoutsetinfo()
+    return res
+
+
 @app.route('/wallets/<wallet_alias>/send')
 @login_required
 def wallet_send(wallet_alias):
@@ -1026,7 +1049,7 @@ def wallet_sendnew(wallet_alias):
                                                     specter=app.specter, rand=rand)
         elif action == "importpsbt":
             try:
-                b64psbt = request.form["rawpsbt"]
+                b64psbt = "".join(request.form["rawpsbt"].split())
                 psbt = wallet.importpsbt(b64psbt)
             except Exception as e:
                 flash("Could not import PSBT: %s" % e, "error")
@@ -1146,6 +1169,12 @@ def wallet_settings(wallet_alias):
             if not res:
                 error="Failed to abort rescan. Maybe already complete?"
             wallet.getdata()
+        elif action == "rescanutxo":
+            wallet.rescanutxo()
+            app.specter._info["utxorescan"] = 1
+        elif action == "abortrescanutxo":
+            app.specter.abortrescanutxo()
+            app.specter._info["utxorescan"] = None
         elif action == "keypoolrefill":
             delta = int(request.form['keypooladd'])
             wallet.keypoolrefill(wallet.keypool, wallet.keypool + delta)
@@ -1153,7 +1182,7 @@ def wallet_settings(wallet_alias):
             wallet.getdata()
         elif action == "deletewallet":
             app.specter.wallet_manager.delete_wallet(
-                wallet, app.specter.bitcoin_datadir
+                wallet, app.specter.bitcoin_datadir, app.specter.chain
             )
             response = redirect(url_for('index'))
             return response
@@ -1255,7 +1284,8 @@ def device(device_alias):
                 app.specter.device_manager.remove_device(
                     device,
                     app.specter.wallet_manager,
-                    bitcoin_datadir=app.specter.bitcoin_datadir
+                    bitcoin_datadir=app.specter.bitcoin_datadir,
+                    chain=app.specter.chain
                 )
                 return redirect("/")
         elif action == "delete_key":
@@ -1294,7 +1324,15 @@ def timedatetime(s):
 @app.template_filter('btcamount')
 def btcamount(value):
     value = float(value)
-    return "{:.8f}".format(value).rstrip("0").rstrip(".")
+    return "{:,.8f}".format(value).rstrip("0").rstrip(".")
+
+
+@app.template_filter('btcunitamount')
+def btcunitamount(value):
+    if app.specter.unit != 'sat':
+        return btcamount(value)
+    value = float(value)
+    return "{:,.0f}".format(round(value * 1e8))
 
 
 @app.template_filter('bytessize')
