@@ -11,13 +11,13 @@ from .helpers import hwi_get_config
 from .specter import Specter
 from .hwi_server import hwi_server
 from .user import User
+from .config import DATA_FOLDER
 
-logger = logging.getLogger(__name__)
+logger = logging.getLogger()
 
 env_path = Path('.') / '.flaskenv'
 load_dotenv(env_path)
 
-DATA_FOLDER = "~/.specter"
 
 def create_app(config="cryptoadvance.specter.config.DevelopmentConfig"):
     # Enables injection of a different config via Env-Variable
@@ -33,31 +33,35 @@ def create_app(config="cryptoadvance.specter.config.DevelopmentConfig"):
         logger.info("pyinstaller based instance running in {}".format(sys._MEIPASS))
         app = Flask(__name__, template_folder=template_folder, static_folder=static_folder)
     else:
-        app = Flask(__name__, template_folder="templates", static_folder="static")
+        app = Flask(
+            __name__,
+            template_folder="templates",
+            static_folder="static"
+        )
     app.config.from_object(config)
     return app
 
 
 def init_app(app, hwibridge=False, specter=None):
     '''  see blogpost 19nd Feb 2020 '''
-    app.logger.info("Initializing QRcode")
     # Login via Flask-Login
     app.logger.info("Initializing LoginManager")
-    if specter == None:
+    if specter is None:
         # the default. If not None, then it got injected for testing
         app.logger.info("Initializing Specter")
         specter = Specter(DATA_FOLDER)
 
     login_manager = LoginManager()
-    login_manager.init_app(app) # Enable Login
-    login_manager.login_view = "login" # Enable redirects if unauthorized
+    login_manager.init_app(app)  # Enable Login
+    login_manager.login_view = "login"  # Enable redirects if unauthorized
+
     @login_manager.user_loader
     def user_loader(id):
         return User.get_user(specter, id)
 
     def login(id):
         login_user(user_loader(id))
-    
+
     app.login = login
     # Attach specter instance so child views (e.g. hwi) can access it
     app.specter = specter
@@ -71,7 +75,7 @@ def init_app(app, hwibridge=False, specter=None):
     if not hwibridge:
         with app.app_context():
             from cryptoadvance.specter import controller
-            if app.config.get("TESTING") and len(app.view_functions) <=3 :
+            if app.config.get("TESTING") and len(app.view_functions) <= 3:
                 # Need to force a reload as otherwise the import is skipped
                 # in pytest, the app is created anew for ech test
                 # But we shouldn't do that if not necessary as this would result in
@@ -84,6 +88,7 @@ def init_app(app, hwibridge=False, specter=None):
             return redirect('/hwi/settings')
     return app
 
+
 def create_and_init():
     ''' This method can be used to fill the FLASK_APP-env variable like
         export FLASK_APP="src/cryptoadvance/specter/server:create_and_init()"
