@@ -9,6 +9,8 @@ from io import BytesIO
 from .specter_error import SpecterError
 import threading
 
+logger = logging.getLogger()
+
 # a gap of 20 addresses is what many wallets do
 WALLET_CHUNK = 20
 wallet_tx_batch = 100
@@ -91,12 +93,15 @@ class Wallet():
         """Checking the gap limit is still ok"""
         if self.last_block is None:
             obj = self.cli.listsinceblock()
-            txs = obj["transactions"]
-            last_block = obj["lastblock"]
         else:
-            obj = self.cli.listsinceblock(self.last_block)
-            txs = obj["transactions"]
-            last_block = obj["lastblock"]
+            # sometimes last_block is invalid, not sure why
+            try:
+                obj = self.cli.listsinceblock(self.last_block)
+            except:
+                logger.error(f"Invalid block {self.last_block}")
+                obj = self.cli.listsinceblock()
+        txs = obj["transactions"]
+        last_block = obj["lastblock"]
         addresses = [tx["address"] for tx in txs]
         # remove duplicates
         addresses = list(dict.fromkeys(addresses))
@@ -172,7 +177,7 @@ class Wallet():
             keys = [Key.from_json(key_dict) for key_dict in wallet_dict['keys']]
             devices = wallet_dict['devices']
         except:
-            Exception('Could not construct a Wallet object from the data provided.')
+            raise Exception('Could not construct a Wallet object from the data provided.')
 
         return cls(
             name,
@@ -220,6 +225,7 @@ class Wallet():
         except:
             # Could happen if address not in wallet (wallet was imported)
             # try adding keypool
+            logger.info(f"Didn't get transactions on address {self.change_address}. Refilling keypool.")
             self.keypoolrefill(0, end=self.keypool, change=False)
             self.keypoolrefill(0, end=self.change_keypool, change=True)
             value_on_address = 0
@@ -321,6 +327,48 @@ class Wallet():
 
         return result
 
+    def rescanutxo(self):
+        t = threading.Thread(target=self._rescan_utxo_thread)
+        t.start()
+
+    def _rescan_utxo_thread(self):
+        args = [
+            "start",
+            [{
+                "desc": self.recv_descriptor,
+                "range": self.keypool
+            },{
+                "desc": self.change_descriptor,
+                "range": self.change_keypool
+            }]
+        ]
+        unspents = self.cli.scantxoutset(*args)["unspents"]
+        res = self.cli.multi([
+            ("getblockhash", tx["height"])
+            for tx in unspents
+        ])
+        block_hashes = [r["result"] for r in res]
+        for i, tx in enumerate(unspents):
+            tx["blockhash"] = block_hashes[i]
+        res = self.cli.multi([
+            ("gettxoutproof", [tx["txid"]], tx["blockhash"])
+            for tx in unspents
+        ])
+        proofs = [r["result"] for r in res]
+        for i, tx in enumerate(unspents):
+            tx["proof"] = proofs[i]
+        res = self.cli.multi([
+            ("getrawtransaction", tx["txid"], False, tx["blockhash"])
+            for tx in unspents
+        ])
+        raws = [r["result"] for r in res]
+        for i, tx in enumerate(unspents):
+            tx["raw"] = raws[i]
+        self.cli.multi([
+            ("importprunedfunds", tx["raw"], tx["proof"])
+            for tx in unspents
+        ])
+
     @property
     def rescan_progress(self):
         """Returns None if rescanblockchain is not launched,
@@ -350,7 +398,7 @@ class Wallet():
 
     @property
     def account_map(self):
-        return '{ "label": "' + self.name + '", "blockheight": ' + str(self.blockheight) + ', "descriptor": "' + self.recv_descriptor.replace("/", "\\/") + '" }'
+        return '{ "label": "' + self.name.replace("'","\\'") + '", "blockheight": ' + str(self.blockheight) + ', "descriptor": "' + self.recv_descriptor.replace("/", "\\/") + '" }'
 
     def getnewaddress(self, change=False):
         label = "Change" if change else "Address"
@@ -378,9 +426,14 @@ class Wallet():
             try:
                 # first try with sortedmulti
                 addr = self.cli.deriveaddresses(desc, [index, index+1])[0]
-            except:
+            except Exception:
                 # if sortedmulti is not supported
-                desc = sort_descriptor(self.cli, desc, index=index, change=change)
+                desc = sort_descriptor(
+                    self.cli,
+                    desc,
+                    index=index,
+                    change=change
+                )
                 addr = self.cli.deriveaddresses(desc)[0]
             return addr
         return self.cli.deriveaddresses(desc, [index, index + 1])[0]
@@ -496,7 +549,7 @@ class Wallet():
         if label == "":
             label = address
         return address_info["label"] if "label" in address_info and address_info["label"] != "" else label
-    
+
     def get_address_name(self, address, addr_idx):
         if self.getlabel(address) == address and addr_idx > -1:
             self.setlabel(address, "Address #{}".format(addr_idx))
@@ -534,7 +587,7 @@ class Wallet():
     @property
     def active_addresses(self):
         return list(dict.fromkeys(self.addresses + self.utxo_addresses))
-    
+
     @property
     def change_addresses(self):
         return [self.get_address(idx, change=True) for idx in range(0, self.change_index + 1)]
@@ -631,6 +684,12 @@ class Wallet():
                 prevtx = CTransaction()
                 prevtx.deserialize(stream)
                 psbt.inputs[i].non_witness_utxo = prevtx
+        else:
+            # remove non_witness_utxo if we don't want them
+            for inp in psbt.inputs:
+                if inp.witness_utxo is not None:
+                    inp.non_witness_utxo = None
+
         if xpubs:
             # for multisig add xpub fields
             if len(self.keys) > 1:
@@ -674,8 +733,8 @@ class Wallet():
         # TODO: check maybe some of the inputs are already locked
         psbt = self.cli.decodepsbt(b64psbt)
         psbt['base64'] = b64psbt
-        amount = 0
-        address = None
+        amount = []
+        address = []
         # get output address and amount
         for out in psbt["tx"]["vout"]:
             if "addresses" not in out["scriptPubKey"] or len(out["scriptPubKey"]["addresses"]) == 0:
@@ -686,13 +745,8 @@ class Wallet():
             # check if it's a change
             if info["iswatchonly"] or info["ismine"]:
                 continue
-            # if not - this is out address
-            # ups, more than one sending address
-            if address is not None:
-                # TODO: we need to have multiple address support 
-                raise SpecterError("Sending to multiple addresses is not supported yet")
-            address = addr
-            amount += out["value"]
+            address.append(addr)
+            amount.append(out["value"])
         # detect signatures
         signed_devices = self.get_signed_devices(psbt)
         psbt["devices_signed"] = [dev.name for dev in signed_devices]
